import json
import pprint as pp
import boto3
from botocore.exceptions import ClientError

def s3_public_policy(s3, bucket_name):
    pub_policy_dict = {
        "Version": "2012-10-17",
        "Statement" : [
            {
                "Sid": "PublicReadGetObject",
                "Effect" : "Allow",
                "Principal": "*",
                "Action": ["s3:GetObject"],
                "Resource" : "arn:aws:s3:::{}/*".format(bucket_name)
            }
        ]
    }

    bucket_policy = json.dumps(pub_policy_dict)

    s3.put_bucket_policy(Bucket=bucket_name, Policy=bucket_policy) ## add policy to s3 bucket

    # get s3 bucket policy after adding new policy
    result = s3.get_bucket_policy(Bucket=bucket_name)

    pp.pprint(result["Policy"])

def s3_website_config(s3, bucket_name):
    # s3 bucket website configs
    website_config = {"IndexDocument" : {"Suffix": "index.html"}}

    s3.put_bucket_website(Bucket = bucket_name,
            WebsiteConfiguration = website_config)

def create_s3_bucket(s3, bucket_name, region):

    # attempt to create bucket
    try:
        location = {'LocationConstraint': region}

        s3.create_bucket(Bucket=bucket_name, CreateBucketConfiguration=location)
    except ClientError as e:
        print("Error: \n\n", e)
        return False
    
    return True  # return true if successful

def upload_file(s3, bucket_name, file_name):

    try: 
        s3.upload_file(file_name, bucket_name, file_name, ExtraArgs={'ContentType':'text/html'})
    except ClientError as e:
        print("Error: \n\n", e)
        return False
    
    return True  # return true if successful

def main():

    # Ask for bucket name
    bucket_name = input("Enter the name of the bucket that will store your website: ")

    # Ask for AWS region, if nothing entered use default region (us-east-1)
    region = input("Enter the name of region your s3 bucket should be in: (Press enter for default us-east-1) ")

    if region is None:
        region = 'us-east-1'

    # create s3 client
    s3_client = boto3.client("s3")

    ## create s3 bucket and add policies/configurations
    if create_s3_bucket(s3_client, bucket_name, region):
        print("successfully created an s3 bucket named {} in region {}".format(bucket_name, region))

        # add policy to s3
        s3_public_policy(s3_client, bucket_name)
        print("\n\n")

        # add website configurations to s3
        s3_website_config(s3_client, bucket_name)
        print("\n\n")

        # upload file to s3
        # ask for name of file to be uploaded
        file_name = input("Enter the name of the files to be uploaded to s3: ")
        upload_file(s3_client, bucket_name, file_name)
        print("successfully uploaded {} to {} bucket".format(file_name, bucket_name))

        print("\n\n Please open the browser and enter the following URL: http://{0}.s3-website.{1}.amazonaws.com".format(bucket_name, region))

if __name__ == "__main__":
    main()